summaryrefslogtreecommitdiff
path: root/src/lib
diff options
context:
space:
mode:
authorCarl Hetherington <cth@carlh.net>2013-04-29 14:46:20 +0100
committerCarl Hetherington <cth@carlh.net>2013-04-29 14:46:20 +0100
commitdbc43b6e3021e34875d7d5bba04abf7ad1fc8633 (patch)
tree38258ef9b5f844e62c6060499f3a2a5cb60b4aa9 /src/lib
parent28dbf4fd074d2046a3c8ddebac9a537a80fd457a (diff)
More cycle-breaking of shared_ptr for decoder / player.
Diffstat (limited to 'src/lib')
-rw-r--r--src/lib/audio_source.cc18
-rw-r--r--src/lib/video_source.cc25
2 files changed, 30 insertions, 13 deletions
diff --git a/src/lib/audio_source.cc b/src/lib/audio_source.cc
index 32b3deccf..2867bcc24 100644
--- a/src/lib/audio_source.cc
+++ b/src/lib/audio_source.cc
@@ -40,13 +40,23 @@ AudioSource::connect_audio (shared_ptr<AudioSink> s)
}
void
-TimedAudioSource::connect_audio (shared_ptr<TimedAudioSink> s)
+TimedAudioSource::connect_audio (shared_ptr<AudioSink> s)
+{
+ Audio.connect (bind (process_audio_proxy, weak_ptr<AudioSink> (s), _1));
+}
+
+static void
+timed_process_audio_proxy (weak_ptr<TimedAudioSink> sink, shared_ptr<const AudioBuffers> audio, double t)
{
- Audio.connect (bind (&TimedAudioSink::process_audio, s, _1, _2));
+ shared_ptr<TimedAudioSink> p = sink.lock ();
+ if (p) {
+ p->process_audio (audio, t);
+ }
}
void
-TimedAudioSource::connect_audio (shared_ptr<AudioSink> s)
+TimedAudioSource::connect_audio (shared_ptr<TimedAudioSink> s)
{
- Audio.connect (bind (&AudioSink::process_audio, s, _1));
+ Audio.connect (bind (timed_process_audio_proxy, weak_ptr<TimedAudioSink> (s), _1, _2));
}
+
diff --git a/src/lib/video_source.cc b/src/lib/video_source.cc
index 2de4db68d..4d505f9fe 100644
--- a/src/lib/video_source.cc
+++ b/src/lib/video_source.cc
@@ -36,22 +36,29 @@ process_video_proxy (weak_ptr<VideoSink> sink, shared_ptr<const Image> i, bool s
void
VideoSource::connect_video (shared_ptr<VideoSink> s)
{
- /* If we bind, say, a Playlist (as the VideoSink) to a Decoder (which is owned
- by the Playlist) we create a cycle. Use a weak_ptr to break it.
+ /* If we bind, say, a Player (as the VideoSink) to a Decoder (which is owned
+ by the Player) we create a cycle. Use a weak_ptr to break it.
*/
- Video.connect (bind (process_video_proxy, boost::weak_ptr<VideoSink> (s), _1, _2, _3));
+ Video.connect (bind (process_video_proxy, weak_ptr<VideoSink> (s), _1, _2, _3));
}
void
-TimedVideoSource::connect_video (shared_ptr<TimedVideoSink> s)
+TimedVideoSource::connect_video (shared_ptr<VideoSink> s)
{
- Video.connect (bind (&TimedVideoSink::process_video, s, _1, _2, _3, _4));
+ Video.connect (bind (process_video_proxy, weak_ptr<VideoSink> (s), _1, _2, _3));
}
-void
-TimedVideoSource::connect_video (shared_ptr<VideoSink> s)
+static void
+timed_process_video_proxy (weak_ptr<TimedVideoSink> sink, shared_ptr<const Image> i, bool same, shared_ptr<Subtitle> s, double t)
{
- Video.connect (bind (&VideoSink::process_video, s, _1, _2, _3));
+ shared_ptr<TimedVideoSink> p = sink.lock ();
+ if (p) {
+ p->process_video (i, same, s, t);
+ }
}
-
+void
+TimedVideoSource::connect_video (shared_ptr<TimedVideoSink> s)
+{
+ Video.connect (bind (timed_process_video_proxy, weak_ptr<TimedVideoSink> (s), _1, _2, _3, _4));
+}